(* ast.mli *)                     
(* @authors: Liming Zhang @version:0.0*)

type op = Add | Sub | Mult | Div | Series | Parallel |Gt |Ne |Lt |Le |Ge | Eq | And | Or
type uop = Reciprocal | Plus | Minus

type afterop = Dot

type datatype = Int | Float | String | Bit | Res | Void | Boolean

type literal = 
    IntLit of int                         (* 42 *)
  | FloatLit of float                     (* 3.4 *)
  | BinaryLit of string                   (* 010101b *)  
  | SentenceLit of string                 (*"Hello World!"*)         
   
type expr =                               (* Expressions *)
    Literal of literal 
  | Id of string                          (* foo *)
  | Binop of expr * op * expr             (* a + b *)
  | Unary of uop * expr                    (* !b *)
  | Afterop of expr * afterop             (* r1.value *)
  | Assign of expr * expr                 (* a = b *)
  | Call of string * (expr list)          (* foo(1, 25) *)
  | Noexpr                                (* While() *)

type var_decl = {
    vtype: datatype;
    vname: string;    
}


type stmt =                               (* Statements  nothing *)
    Block of (var_decl list) * (stmt list)
  | Expr of expr                          (* foo = bar + 3; *)
  | Return of expr                        (* return 42 *)                      
  | If of expr * stmt * stmt              (* if (foo == 42) {} else {} *)
  | For of expr * expr * expr * stmt      (* for loop *)
  | While of expr * stmt                  (* while (i<10) { i = i + 1 } *)



type func_decl = {
    rtype : datatype;
    fname : string;
    formals : var_decl list;
    locals : var_decl list;
    body : stmt list;
  }

type program = var_decl list  * func_decl list
