(* ast.mli *)                     
(* @authors: Liming Zhang @version:0.0*)
open Ast
	
	
type var_decl = {
    vvtype: datatype;
    vvname: string;    
}
	
type simple_expr =                               (* Expressions *)
    Literal of literal 
  | Id of var_decl                          (* foo *)
  | Binop of expr * op * expr             (* a + b *)
  | Unary of uop * expr                    (* !b *)
  | Afterop of expr * afterop             (* r1.value *)
  | Assign of expr * expr                 (* a = b *)
  | Call of string * (expr list)          (* foo(1, 25) *)
  | Noexpr                                (* While() *)
and expr = simple_expr * datatype

type stmt =                               (* Statements  nothing *)
    Block of (var_decl list) * (stmt list)
  | Expr of expr                          (* foo = bar + 3; *)
  | Return of expr                        (* return 42 *)                      
  | If of expr * stmt * stmt              (* if (foo == 42) {} else {} *)
  | For of expr * expr * expr * stmt      (* for loop *)
  | While of expr * stmt                  (* while (i<10) { i = i + 1 } *)


type func_decl = {
    frtype : datatype;
    ffname : string;
    fformals : var_decl list;
    flocals : var_decl list;
    fbody : stmt list;
  }

type program = var_decl list  * func_decl list

